@@ -2,10 +2,13 @@ from collections import defaultdict
 import inspect
 from itertools import chain
 import re
+import string
 from typing import Any, Callable, DefaultDict, Dict, List, Type, Union
+import unicodedata
 
-from statham.dsl.constants import NotPassed
+from statham.dsl.constants import COMPOSITION_KEYWORDS, NotPassed
 from statham.dsl.elements import (
+    AllOf,
     AnyOf,
     Array,
     Boolean,
@@ -25,7 +28,7 @@ from statham.dsl.elements.meta import (
     RESERVED_PROPERTIES,
 )
 from statham.dsl.exceptions import FeatureNotImplementedError, SchemaParseError
-from statham.dsl.helpers import reraise
+from statham.dsl.helpers import expand, reraise, split_dict
 from statham.dsl.property import _Property
 
 
@@ -95,7 +98,7 @@ def parse_element(schema: Dict[str, Any], state: ParseState = None) -> Element:
     if "items" in schema:
         schema["items"] = parse_items(schema, state)
     schema["additionalProperties"] = parse_additional_properties(schema, state)
-    if {"anyOf", "oneOf"} & set(schema):
+    if {"anyOf", "oneOf", "allOf"} & set(schema):
         return parse_composition(schema, state)
     if "type" not in schema:
         return Element(**keyword_filter(Element)(schema))
@@ -104,33 +107,42 @@ def parse_element(schema: Dict[str, Any], state: ParseState = None) -> Element:
 
 def parse_composition(
     schema: Dict[str, Any], state: ParseState = None
-) -> CompositionElement:
+) -> Element:
     """Parse a schema with composition keywords.
 
-    :raises ValueError: if the schema does not contain any composition
-        keywords.
-    :raises FeatureNotImplementedError: if multiple composition keywords
-        are present.
+    Handles multiple composition keywords by wrapping them in an AllOf
+    element. Similarly, non-keyword elements are parsed as usual and
+    included in an allOf element.
+
+    For example:
+    ```python
+    schema = {
+        "type": "integer",
+        "oneOf": [{"minimum": 3}, {"maximum": 5}],
+        "anyOf": [{"multipleOf": 2}, {"multipleOf": 3}],
+    }
+    parse_composition(schema) == AllOf(
+        Integer(),
+        OneOf(Element(minimum=3), Element(maximum=5)),
+        AnyOf(Element(multipleOf=2), Element(multipleOf=3)),
+    )
+    ```
     """
     state = state or ParseState()
-    intersect = {"anyOf", "oneOf"} & set(schema)
-    element_type: Type[CompositionElement]
-    if intersect == {"anyOf"}:
-        element_type = AnyOf
-        sub_schemas = schema["anyOf"]
-    elif intersect == {"oneOf"}:
-        element_type = OneOf
-        sub_schemas = schema["oneOf"]
-    elif not intersect:
-        raise ValueError(
-            "Schema passed to `parse_composition` has no supported "
-            "validation keywords."
-        )
-    else:
-        raise FeatureNotImplementedError.multiple_composition_keywords()
-    return element_type(
-        *(parse_element(sub_schema, state) for sub_schema in sub_schemas)
+    composition, other = split_dict(set(COMPOSITION_KEYWORDS) | {"default"})(
+        schema
     )
+    base_element = parse_element(other, state)
+    for key in COMPOSITION_KEYWORDS:
+        composition[key] = [
+            parse_element(sub_schema) for sub_schema in composition.get(key, [])
+        ]
+    all_of = [base_element] + composition["allOf"]
+    all_of.append(_compose_elements(OneOf, composition["oneOf"]))
+    all_of.append(_compose_elements(AnyOf, composition["anyOf"]))
+    element = _compose_elements(AllOf, all_of)
+    element.default = composition.get("default") or element.default
+    return element
 
 
 def parse_typed(
@@ -206,6 +218,13 @@ def parse_object(
     title = _title_format(title)
     default = schema.get("default", NotPassed())
     properties = schema.get("properties", {})
+    properties.update(
+        {
+            key: _Property(Element(), required=True)
+            for key in schema.get("required", [])
+            if key not in properties
+        }
+    )
     class_dict = ObjectClassDict(
         default=default,
         options=ObjectOptions(**keyword_filter(ObjectOptions)(schema)),
@@ -222,27 +241,55 @@ def parse_properties(
     """Parse properties from a schema element."""
     state = state or ParseState()
     required = set(schema.get("required", []))
-    attr_name = lambda key: key if key not in RESERVED_PROPERTIES else f"_{key}"
+    properties = schema.get("properties", {})
     return {
         **{
-            # TODO: Handle attribute names which don't work in python.
-            attr_name(key): _Property(
+            parse_attribute_name(key): _Property(
                 parse_element(value, state),
                 required=key in required,
                 source=key,
             )
-            for key, value in schema.get("properties", {}).items()
+            for key, value in properties.items()
             # Ignore malformed values.
             if isinstance(value, dict)
         },
         **{
-            attr_name(key): prop
-            for key, prop in schema.get("properties", {}).items()
+            parse_attribute_name(key): prop
+            for key, prop in properties.items()
             if isinstance(prop, _Property)
         },
     }
 
 
+def parse_attribute_name(name: str) -> str:
+    """Convert attibute name to valid python attribute.
+
+    Attempts to replace special characters with their unicode names.
+    """
+
+    def _char_map(idx: int, char: str) -> str:
+        if char.isalnum() or char in ("_", "-", " "):
+            return char
+        if char in string.whitespace:
+            return "_"
+        label = unicodedata.name(char, "unknown").lower()
+        if idx != 0 and name[idx - 1] != "_":
+            label = "_" + label
+        if idx != len(name) - 1 and name[idx + 1] != "_":
+            label = label + "_"
+        return label
+
+    chars = map(expand(_char_map), enumerate(name))
+    name = "".join(chars).replace(" ", "_").replace("-", "_")
+    if not name:
+        return "blank"
+    return (
+        name
+        if name not in RESERVED_PROPERTIES and name[0].isalpha()
+        else f"_{name}"
+    )
+
+
 def parse_additional_properties(
     schema: Dict[str, Any], state: ParseState = None
 ) -> Union[Element, bool]:
@@ -276,6 +323,22 @@ def parse_items(schema: Dict[str, Any], state: ParseState = None) -> Element:
     return parse_element(items, state)
 
 
+def _compose_elements(
+    element_type: Type[CompositionElement], elements: List[Element]
+) -> Element:
+    """Create a composition element from a type and list of component elements.
+
+    Filters out trivial elements, and simplifies compositions with only one
+    composed element.
+    """
+    elements = [elem for elem in elements if elem != Element()]
+    if not elements:
+        return Element()
+    if len(elements) == 1:
+        return elements[0]
+    return element_type(*elements)
+
+
 def keyword_filter(type_: Type) -> Callable[[Dict[str, Any]], Dict[str, Any]]:
     """Create a filter to pull out only relevant keywords for a given type."""
     params = inspect.signature(type_.__init__).parameters.values()
@@ -287,9 +350,9 @@ def keyword_filter(type_: Type) -> Callable[[Dict[str, Any]], Dict[str, Any]]:
     return _filter
 
 
-def _title_format(string: str) -> str:
+def _title_format(name: str) -> str:
     """Convert titles in schemas to class names."""
-    words = list(filter(None, re.split(r"[ _-]", string)))
+    words = list(filter(None, re.split(r"[ _-]", name)))
     segments = chain.from_iterable(
         [
             re.findall("[A-Z][^A-Z]*", word[0].upper() + word[1:])
